#!/usr/bin/perl

use Nagios::Plugin::DieNicely;

#
# Your Nagios plugin code goes here. If the code in 
# the modules you use, or your own code should issue a die
# croak, or confess, Nagios will recieve the output in 
# the form of a CRITICAL
#

